@@ -2,6 +2,7 @@
 import random
 import math
 import numpy as np
+import warnings
 
 from qlazy.config import *
 from qlazy.error import *
@@ -47,7 +48,7 @@ class DensOp(ctypes.Structure):
         >>> import numpy as np
         >>> qs_0 = QState(2)
         >>> qs_1 = QState(2).h(0).cx(0,1)
-        >>> de_A = DensOp(qubit_num=2)  # = |00..0><00..0|
+        >>> de_A = DensOp(qubit_num=5)  # = |00..0><00..0|
         >>> de_B = DensOp(qstate=[qs_0,qs_1], prob=[0.2,0.8])
         >>> de_C = DensOp(matrix=np.array([[0.7,0.0],[0.0,0.3]]))
 
@@ -66,7 +67,7 @@ class DensOp(ctypes.Structure):
             qstate = [QState(qubit_num=qubit_num)]
             prob = [1.0]
             de = densop_init(qstate, prob)
-            qstate[0].free()
+            # qstate[0].free()
             return de
         
         elif qstate != [] and prob != []:
@@ -120,7 +121,7 @@ class DensOp(ctypes.Structure):
             de_tmp = densop[i].clone()
             de_tmp.mul(factor=prob[i])
             de_out.add(densop=de_tmp)
-            de_tmp.free()
+            # de_tmp.free()
 
         return de_out
 
@@ -252,12 +253,37 @@ class DensOp(ctypes.Structure):
         -------
         None
 
+        """
+        # for de in densops:
+        #     if type(de) is list or type(de) is tuple:
+        #         cls.free_all(*de)
+        #     elif type(de) is DensOp:
+        #         de.free()
+        #     else:
+        #         raise DensOp_Error_FreeAll()
+        
+        warnings.warn("No need to call 'free_all' method because free automatically, or you can use class method 'del_all' to free memory explicitly.")
+
+    @classmethod
+    def del_all(cls, *densops):
+        """
+        free memory of the all density operators.
+
+        Parameters
+        ----------
+        densops : instance of DensOp,instance of DensOp,...
+            set of DensOp instances
+
+        Returns
+        -------
+        None
+
         """
         for de in densops:
             if type(de) is list or type(de) is tuple:
-                cls.free_all(*de)
+                cls.del_all(*de)
             elif type(de) is DensOp:
-                de.free()
+                del de
             else:
                 raise DensOp_Error_FreeAll()
 
@@ -290,7 +316,7 @@ class DensOp(ctypes.Structure):
         """
         de_part = self.partial(qid=qid)
         elm = densop_get_elm(de_part)
-        de_part.free()
+        # de_part.free()
         return elm
     
     def show(self, qid=[], nonzero=False):
@@ -332,7 +358,7 @@ class DensOp(ctypes.Structure):
         """
         de_part = self.partial(qid=qid)
         densop_print(de_part, nonzero)
-        de_part.free()
+        # de_part.free()
 
     def clone(self):
         """
@@ -511,11 +537,34 @@ class DensOp(ctypes.Structure):
             de = self.clone()
             for i in range(num-1):
                 de_tmp = de.tenspro(self)
-                de.free()
+                # de.free()
                 de = de_tmp.clone()
-                de_tmp.free()
+                # de_tmp.free()
             return de
         
+    def join(self, de_list):
+        """
+        get tensor product state (density operator) of the density operators' list.
+
+        Parameters
+        ----------
+        de_list : list (DensOp)
+            list of density operators.
+
+        Returns
+        -------
+        de_out : instance of DensOp
+            tensor product state (density operator).
+
+        """
+        de_out = self.clone()
+        for de in de_list:
+            de_tmp = de_out.clone()
+            # de_out.free()
+            de_out = de_tmp.tenspro(de)
+            # de_tmp.free()
+        return de_out
+
     def expect(self, matrix=None):
         """
         get the expectation value of matrix under this density operator.
@@ -539,7 +588,7 @@ class DensOp(ctypes.Structure):
         densop = self.clone()
         densop_apply_matrix(densop, matrix=matrix, dire='left')
         value = densop.trace()
-        densop.free()
+        # densop.free()
         return value
         
     def apply(self, matrix=None, qid=[], dire='both'):
@@ -664,8 +713,8 @@ class DensOp(ctypes.Structure):
                     densop_tmp = densop_ori.clone()
                     densop_tmp.apply(matrix=kraus[i], qid=qid, dire='both')
                     self.add(densop=densop_tmp)
-                    densop_tmp.free()
-            densop_ori.free()
+                    # densop_tmp.free()
+            # densop_ori.free()
                 
         else:  # selective measurement
 
@@ -973,7 +1022,7 @@ class DensOp(ctypes.Structure):
             else:
                 de_part = self.partial(qid=qid)
                 ent = de_part.__von_neumann_entropy()
-                de_part.free()
+                # de_part.free()
                 
         return ent
 
@@ -1843,6 +1892,52 @@ class DensOp(ctypes.Structure):
         densop_mcx(self, qid)
         return self
 
+    def operate(self, pp=None, ctrl=None):
+        """
+        operate unitary operator to density operator.
+
+        Parameters
+        ----------
+        pp : instance of PauliProduct
+            pauli product to operate
+        ctrl : int
+            contoroll qubit id for controlled pauli product
+
+        Returns
+        -------
+        self : instance of DensOp
+            density operator after operation
+
+        """
+        pauli_list = pp.pauli_list
+        qid = pp.qid
+    
+        if ctrl is None:
+            for q, pauli in zip(qid, pauli_list):
+                if pauli == 'X':
+                    self.x(q)
+                elif pauli == 'Y':
+                    self.y(q)
+                elif pauli == 'Z':
+                    self.z(q)
+                else:
+                    continue
+        else:
+            if ctrl in qid:
+                raise ValueError("controll and target qubit id conflict")
+        
+            for q, pauli in zip(qid, pauli_list):
+                if pauli == 'X':
+                    self.cx(ctrl, q)
+                elif pauli == 'Y':
+                    self.cy(ctrl, q)
+                elif pauli == 'Z':
+                    self.cz(ctrl, q)
+                else:
+                    continue
+    
+        return self
+
     def free(self):
         """
         free memory of quantum state.
@@ -1857,9 +1952,12 @@ class DensOp(ctypes.Structure):
         self : instance of DensOp
 
         """
-        # return self.densop_free()
-        return densop_free(self)
+        # densop_free(self)
+        warnings.warn("No need to call 'free' method because free automatically, or you can use 'del' to free memory explicitly.")
 
+    def __del__(self):
+
+        densop_free(self)
     
 # c-library for densop
 from qlazy.lib.densop_c import *
