@@ -17,8 +17,18 @@ class QComp:
         size of classical memory
     cmem : list
         classical memory
-    qcirc : dict
+    qcirc : list of dict
         quantum circuit
+        dict = {'kind': kind, 'qid': qid, 'cid': cid, 
+                'phase': phase, 'phase1': phase1, 'phase2': phase2,
+                'ctrl': ctrl}
+        kind: int
+        qid: list of int
+        cid: list of int
+        phase: float
+        phase1: float
+        phase2: float
+        ctrl: int
     qstate : instance of quantum state
         quantum state (depend on backend)
 
@@ -88,45 +98,45 @@ class QComp:
             self.__reset = reset
             self.__free = free
 
+        # # 
+        # # note: followings are not supported in the near future
+        # #
         # 
-        # note: followings are not supported in the near future
-        #
-        
-        # qlazy qstate simulator
-        elif self.backend.name == 'qlazy_qstate_simulator':
-            from qlazy.backend.qlazy_qstate_simulator import init, run, reset, free
-            self.__init = init
-            self.__run = run
-            self.__reset = reset
-            self.__free = free
-            warnings.warn("You should use name='qlazy',device='qstate_simulator', because name='qlazy_qstate_simulator' will be not supported in the near future")
-        
-        # qlazy stabilizer simulator
-        elif self.backend.name == 'qlazy_stabilizer_simulator':
-            from qlazy.backend.qlazy_stabilizer_simulator import init, run, reset, free
-            self.__init = init
-            self.__run = run
-            self.__reset = reset
-            self.__free = free
-            warnings.warn("You should use name='qlazy',device='stabilizer_simulator', because name='qlazy_stabilizer_simulator' will be not supported in the near future")
-        
-        # qulacs
-        elif self.backend.name == 'qulacs_simulator':
-            from qlazy.backend.qulacs_cpu_simulator import init, run, reset, free
-            self.__init = init
-            self.__run = run
-            self.__reset = reset
-            self.__free = free
-            warnings.warn("You should use name='qulacs',device='cpu_simulator', because name='qulacs_simulator' will be not supported in the near future")
-        
-        # qulacs-gpu
-        elif self.backend.name == 'qulacs_gpu_simulator':
-            from qlazy.backend.qulacs_gpu_simulator import init, run, reset, free
-            self.__init = init
-            self.__run = run
-            self.__reset = reset
-            self.__free = free
-            warnings.warn("You should use name='qulacs',device='gpu_simulator', because name='qulacs_gpu_simulator' will be not supported in the near future")
+        # # qlazy qstate simulator
+        # elif self.backend.name == 'qlazy_qstate_simulator':
+        #     from qlazy.backend.qlazy_qstate_simulator import init, run, reset, free
+        #     self.__init = init
+        #     self.__run = run
+        #     self.__reset = reset
+        #     self.__free = free
+        #     warnings.warn("You should use name='qlazy',device='qstate_simulator', because name='qlazy_qstate_simulator' will be not supported in the near future")
+        # 
+        # # qlazy stabilizer simulator
+        # elif self.backend.name == 'qlazy_stabilizer_simulator':
+        #     from qlazy.backend.qlazy_stabilizer_simulator import init, run, reset, free
+        #     self.__init = init
+        #     self.__run = run
+        #     self.__reset = reset
+        #     self.__free = free
+        #     warnings.warn("You should use name='qlazy',device='stabilizer_simulator', because name='qlazy_stabilizer_simulator' will be not supported in the near future")
+        # 
+        # # qulacs
+        # elif self.backend.name == 'qulacs_simulator':
+        #     from qlazy.backend.qulacs_cpu_simulator import init, run, reset, free
+        #     self.__init = init
+        #     self.__run = run
+        #     self.__reset = reset
+        #     self.__free = free
+        #     warnings.warn("You should use name='qulacs',device='cpu_simulator', because name='qulacs_simulator' will be not supported in the near future")
+        # 
+        # # qulacs-gpu
+        # elif self.backend.name == 'qulacs_gpu_simulator':
+        #     from qlazy.backend.qulacs_gpu_simulator import init, run, reset, free
+        #     self.__init = init
+        #     self.__run = run
+        #     self.__reset = reset
+        #     self.__free = free
+        #     warnings.warn("You should use name='qulacs',device='gpu_simulator', because name='qulacs_gpu_simulator' will be not supported in the near future")
         
         else:
             raise Backend_Error_NameNotSupported()
@@ -177,12 +187,70 @@ class QComp:
         None
 
         """
+        # if self.qstate != None:
+        #     self.__free(qstate=self.qstate, backend=self.backend)
+        # 
+        # del self.cmem
+        # del self.qcirc
+        
+        warnings.warn("No need to call 'free' method because free automatically, or you can use 'del' to free memory explicitly.")
+
+    def __del__(self):
+        
         if self.qstate != None:
             self.__free(qstate=self.qstate, backend=self.backend)
 
         del self.cmem
         del self.qcirc
 
+    @classmethod
+    def free_all(cls, *qcomps):
+        """
+        free memory of the all quantum computers.
+
+        Parameters
+        ----------
+        qcomps : instance of QComp,instance of QComp,...
+            set of QComp instances
+
+        Returns
+        -------
+        None
+
+        """
+        # for qc in qcomps:
+        #     if type(qc) is list or type(qc) is tuple:
+        #         cls.free_all(*qc)
+        #     elif type(qc) is QComp:
+        #         qc.free()
+        #     else:
+        #         raise QComp_Error_FreeAll()
+        
+        warnings.warn("No need to call 'free_all' method because free automatically, or you can use class method 'del_all' to free memory explicitly.")
+
+    @classmethod
+    def del_all(cls, *qcomps):
+        """
+        free memory of the all quantum computers.
+
+        Parameters
+        ----------
+        qcomps : instance of QComp,instance of QComp,...
+            set of QComp instances
+
+        Returns
+        -------
+        None
+
+        """
+        for qc in qcomps:
+            if type(qc) is list or type(qc) is tuple:
+                cls.del_all(*qc)
+            elif type(qc) is QComp:
+                del qc
+            else:
+                raise QComp_Error_FreeAll()
+
     def run(self, shots=DEF_SHOTS, reset_qubits=True, reset_cmem=True, reset_qcirc=True):
         """
         run the quantum circuit.
@@ -1070,6 +1138,52 @@ class QComp:
         self.cx(q2,q1,ctrl=ctrl).ccx(q0,q1,q2,ctrl=ctrl).cx(q2,q1,ctrl=ctrl)
         return self
 
+    def operate(self, pp=None, ctrl=None):
+        """
+        add unitary operator.
+
+        Parameters
+        ----------
+        pp : instance of PauliProduct
+            pauli product to operate
+        ctrl : int
+            contoroll qubit id for controlled pauli product
+
+        Returns
+        -------
+        self : instance of QComp
+            quantum computer after adding
+
+        """
+        pauli_list = pp.pauli_list
+        qid = pp.qid
+    
+        if ctrl is None:
+            for q, pauli in zip(qid, pauli_list):
+                if pauli == 'X':
+                    self.x(q)
+                elif pauli == 'Y':
+                    self.y(q)
+                elif pauli == 'Z':
+                    self.z(q)
+                else:
+                    continue
+        else:
+            if ctrl in qid:
+                raise ValueError("controll and target qubit id conflict")
+        
+            for q, pauli in zip(qid, pauli_list):
+                if pauli == 'X':
+                    self.cx(ctrl, q)
+                elif pauli == 'Y':
+                    self.cy(ctrl, q)
+                elif pauli == 'Z':
+                    self.cz(ctrl, q)
+                else:
+                    continue
+    
+        return self
+
     def __add_quantum_gate(self, kind=None, qid=None, cid=None,
                            phase=DEF_PHASE, phase1=DEF_PHASE, phase2=DEF_PHASE,
                            ctrl=None):
