@@ -2,11 +2,13 @@
 import random
 import numpy as np
 from collections import Counter
+import warnings
 
 from qlazy.config import *
 from qlazy.error import *
 from qlazy.MData import *
 from qlazy.Observable import *
+from qlazy.PauliProduct import *
 from qlazy.lib.qstate_mcx import *
 
 MDATA_TABLE = {}
@@ -28,8 +30,10 @@ class QState(ctypes.Structure):
     _fields_ = [
         ('qubit_num', ctypes.c_int),
         ('state_num', ctypes.c_int),
+        ('buf_id', ctypes.c_int),
         ('camp', ctypes.c_void_p),
-        ('camp_tmp', ctypes.c_void_p),
+        ('buffer_0', ctypes.c_void_p),
+        ('buffer_1', ctypes.c_void_p),
         ('gbank', ctypes.c_void_p),
     ]
 
@@ -61,7 +65,7 @@ class QState(ctypes.Structure):
 
         else:
             return qstate_init_with_vector(vector, seed)
-            
+
     def __str__(self):
 
         return str(self.get_amp())
@@ -99,6 +103,7 @@ class QState(ctypes.Structure):
         --------
         >>> def bell(self, q0, q1):
         >>>     self.h(q0).cx(q0,q1)
+        >>>     return self
         >>> ...
         >>> QState.add_method(bell)
         >>> qs = QState(qubit_num=2)
@@ -207,12 +212,37 @@ class QState(ctypes.Structure):
         -------
         None
 
+        """
+        # for qs in qstates:
+        #     if type(qs) is list or type(qs) is tuple:
+        #         cls.free_all(*qs)
+        #     elif type(qs) is QState:
+        #         qs.free()
+        #     else:
+        #         raise QState_Error_FreeAll()
+
+        warnings.warn("No need to call 'free_all' method because free automatically, or you can use class method 'del_all' to free memory explicitly.")
+        
+    @classmethod
+    def del_all(cls, *qstates):
+        """
+        free memory of the all quantum states.
+
+        Parameters
+        ----------
+        qstates : instance of QState,instance of QState,...
+            set of QState instances
+
+        Returns
+        -------
+        None
+
         """
         for qs in qstates:
             if type(qs) is list or type(qs) is tuple:
-                cls.free_all(*qs)
+                cls.del_all(*qs)
             elif type(qs) is QState:
-                qs.free()
+                del qs
             else:
                 raise QState_Error_FreeAll()
 
@@ -392,8 +422,8 @@ class QState(ctypes.Structure):
             qs_0 = self.partial(qid=qid)
             qs_1 = qstate.partial(qid=qid)
             inp = qstate_inner_product(qs_0, qs_1)
-            qs_0.free()
-            qs_1.free()
+            # qs_0.free()
+            # qs_1.free()
         return inp
         
     def tenspro(self, qstate):
@@ -462,11 +492,34 @@ class QState(ctypes.Structure):
             qs = self.clone()
             for i in range(num-1):
                 qs_tmp = qs.tenspro(self)
-                qs.free()
+                # qs.free()
                 qs = qs_tmp.clone()
-                qs_tmp.free()
+                # qs_tmp.free()
             return qs
         
+    def join(self, qs_list):
+        """
+        get tensor product state of the quantum states' list.
+
+        Parameters
+        ----------
+        qs_list : list (QState)
+            list of quantum states.
+
+        Returns
+        -------
+        qs_out : instance of QState
+            tensor product state.
+
+        """
+        qs_out = self.clone()
+        for qs in qs_list:
+            qs_tmp = qs_out.clone()
+            # qs_out.free()
+            qs_out = qs_tmp.tenspro(qs)
+            # qs_tmp.free()
+        return qs_out
+
     def evolve(self, observable=None, time=0.0, iter=0):
         """
         evolve the quantum state.
@@ -1605,6 +1658,52 @@ class QState(ctypes.Structure):
         mfrq = MDATA_TABLE[tag_long].measured_freq(angle=angle, phase=phase)
         return mfrq
 
+    def operate(self, pp=None, ctrl=None):
+        """
+        operate unitary operator to quantum state.
+
+        Parameters
+        ----------
+        pp : instance of PauliProduct
+            pauli product to operate
+        ctrl : int
+            contoroll qubit id for controlled pauli product
+
+        Returns
+        -------
+        self : instance of QState
+            quantum state after operation
+
+        """
+        pauli_list = pp.pauli_list
+        qid = pp.qid
+    
+        if ctrl is None:
+            for q, pauli in zip(qid, pauli_list):
+                if pauli == 'X':
+                    self.x(q)
+                elif pauli == 'Y':
+                    self.y(q)
+                elif pauli == 'Z':
+                    self.z(q)
+                else:
+                    continue
+        else:
+            if ctrl in qid:
+                raise ValueError("controll and target qubit id conflict")
+        
+            for q, pauli in zip(qid, pauli_list):
+                if pauli == 'X':
+                    self.cx(ctrl, q)
+                elif pauli == 'Y':
+                    self.cy(ctrl, q)
+                elif pauli == 'Z':
+                    self.cz(ctrl, q)
+                else:
+                    continue
+    
+        return self
+
     def free(self):
         """
         free memory of quantum state.
@@ -1618,8 +1717,12 @@ class QState(ctypes.Structure):
         None
 
         """
+        # qstate_free(self)
+        warnings.warn("No need to call 'free' method because free automatically, or you can use 'del' to free memory explicitly.")
+
+    def __del__(self):
+
         qstate_free(self)
-        
 
 # c-library for qstate
 from qlazy.lib.qstate_c import *
