@@ -5,8 +5,12 @@ from .DensOp import DensOp
 from .Stabilizer import Stabilizer
 from .QComp import QComp
 from .Backend import Backend
+from .QCirc import QCirc
+from .CMem import CMem
+from .PauliProduct import PauliProduct
 from . import config
 from . import error
 from . import util
 
-__all__ = ["QState","Observable","DensOp","Stabilizer","Qcomp","Backend","config","error","util"]
+__all__ = ["QState","Observable","DensOp","Stabilizer","Qcomp","Backend","QCirc","CMem",
+           "UnitaryOperator","PauliProduct","config","error","util"]
